// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use crate::auth::application::port::input::forge::refresh_access_token::RefreshAccessTokenInputPort;
use crate::auth::application::port::out::db::{
    delete_oauth_access_token::DeleteOAuthAccessToken, get_oauth_access_token::GetOAuthAccessToken,
    save_oauth_access_token::SaveOAuthAccessToken,
};
use crate::auth::application::port::out::forge::refresh_access_token::RefreshAccessToken;
use crate::auth::application::services::refresh_access_token::{
    command::*, errors::*, service::*, RefreshAccessTokenUseCase,
};

#[derive(Clone)]
pub struct RefreshAccessTokenInputAdapter {
    save_oauth_access_token_adapter: Arc<dyn SaveOAuthAccessToken>,
    get_oauth_access_token_adapter: Arc<dyn GetOAuthAccessToken>,
    delete_oauth_access_token_adapter: Arc<dyn DeleteOAuthAccessToken>,
}

impl RefreshAccessTokenInputAdapter {
    pub fn new(
        save_oauth_access_token_adapter: Arc<dyn SaveOAuthAccessToken>,
        get_oauth_access_token_adapter: Arc<dyn GetOAuthAccessToken>,
        delete_oauth_access_token_adapter: Arc<dyn DeleteOAuthAccessToken>,
    ) -> Self {
        Self {
            save_oauth_access_token_adapter,
            get_oauth_access_token_adapter,
            delete_oauth_access_token_adapter,
        }
    }
}

#[async_trait::async_trait]
impl RefreshAccessTokenInputPort for RefreshAccessTokenInputAdapter {
    async fn refresh_access_token(
        &self,
        refresh_access_token_adapter: Arc<dyn RefreshAccessToken>,
        cmd: RefreshAccessTokenCommand,
    ) -> RefreshAccessTokenServiceResult<()> {
        let s = RefreshAccessTokenService::new(
            self.save_oauth_access_token_adapter.clone(),
            self.get_oauth_access_token_adapter.clone(),
            self.delete_oauth_access_token_adapter.clone(),
            refresh_access_token_adapter,
        );
        s.refresh_access_token(cmd).await?;
        Ok(())
    }
}
