// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use actix_web::web;

pub mod login;
mod process_authorization;
mod routes;
mod template;
pub mod types;

use routes::RoutesRepository;

pub fn load_ctx() -> impl FnOnce(&mut web::ServiceConfig) {
    let routes = types::WebRouteRepository::new(Arc::new(RoutesRepository::default()));

    let f = move |cfg: &mut web::ServiceConfig| {
        cfg.app_data(routes);
        cfg.configure(login::services);
        cfg.configure(process_authorization::services);
    };

    Box::new(f)
}
