// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use crate::auth::adapter::out::forge::SupportedForges;

#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
pub struct RoutesRepository {
    process_oauth_authorization_response: String,
    pub login: String,
    oauth_login: String,
}

impl Default for RoutesRepository {
    fn default() -> Self {
        Self {
            process_oauth_authorization_response: "/oauth/{forge}/authorize".to_string(),
            login: "/login".to_string(),
            oauth_login: "/oauth/{forge}/login".to_string(),
        }
    }
}

impl RoutesRepository {
    pub fn oauth_login(&self, forge_name: &SupportedForges) -> String {
        self.oauth_login.replace("{forge}", &forge_name.to_string())
    }
    pub fn process_oauth_authorization_response(&self, forge_name: &SupportedForges) -> String {
        self.process_oauth_authorization_response
            .replace("{forge}", &forge_name.to_string())
    }
}
