// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use lazy_static::lazy_static;
use rust_embed::RustEmbed;
use std::cell::RefCell;
use tera::*;

use serde::Serialize;

pub struct TemplateFile {
    pub name: &'static str,
    pub path: &'static str,
}

impl TemplateFile {
    pub const fn new(name: &'static str, path: &'static str) -> Self {
        Self { name, path }
    }

    pub fn register(&self, t: &mut Tera) -> std::result::Result<(), tera::Error> {
        t.add_raw_template(self.name, &Templates::get_template(self).expect(self.name))
    }

    //    #[cfg(test)]
    //    #[allow(dead_code)]
    //    pub fn register_from_file(&self, t: &mut Tera) -> std::result::Result<(), tera::Error> {
    //        use std::path::Path;
    //        t.add_template_file(Path::new("./").join(self.path), Some(self.name))
    //    }
}

pub const PAYLOAD_KEY: &str = "payload";

//pub const BASE: TemplateFile = TemplateFile::new("base", "components/base.html");
//pub const FOOTER: TemplateFile = TemplateFile::new("footer", "components/footer.html");
//pub const PUB_NAV: TemplateFile = TemplateFile::new("pub_nav", "components/nav/pub.html");
//pub const AUTH_NAV: TemplateFile = TemplateFile::new("auth_nav", "components/nav/auth.html");

lazy_static! {
    pub static ref TEMPLATES: Tera = {
        let mut tera = Tera::default();
//        for t in [BASE, FOOTER, PUB_NAV, AUTH_NAV].iter() {
//            t.register(&mut tera).unwrap();
//        }
//        errors::register_templates(&mut tera);
        tera.autoescape_on(vec![".html", ".sql"]);
        super::login::register_templates(&mut tera);
        tera
    };
}

#[derive(RustEmbed)]
#[folder = "src/auth/adapter/input/web/"]
#[include = "*.html"]
#[exclude = "*.rs"]
struct Templates;

impl Templates {
    fn get_template(t: &TemplateFile) -> Option<String> {
        match Self::get(t.path) {
            Some(file) => Some(String::from_utf8_lossy(&file.data).into_owned()),
            None => None,
        }
    }
}

pub fn tera_context<T: Serialize>(s: &T) -> RefCell<Context> {
    let c = RefCell::new(Context::new());
    c.borrow_mut().insert(PAYLOAD_KEY, s);
    c
}
