// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use actix_web::web;

use crate::auth::adapter::out::forge::forge_repository::ForgeRepositoryInterface;
use crate::auth::application::port::out::db::{
    delete_oauth_access_token::DeleteOAuthAccessToken, delete_oauth_state::DeleteOAuthState,
    get_oauth_access_token::GetOAuthAccessToken, oauth_state_exists::OAuthStateExists,
    save_oauth_access_token::SaveOAuthAccessToken, save_oauth_state::SaveOAuthState,
};
pub(super) use crate::utils::random_string::WebGenerateRandomStringInterface;

use super::RoutesRepository;

pub type WebForgeRepositoryInterface = web::Data<Arc<dyn ForgeRepositoryInterface>>;

pub type WebRouteRepository = web::Data<Arc<RoutesRepository>>;

pub type WebSettings = web::Data<crate::settings::Settings>;

pub type WebSaveOauthState = web::Data<Arc<dyn SaveOAuthState>>;
pub type WebOauthStateExists = web::Data<Arc<dyn OAuthStateExists>>;
pub type WebDeleteOauthState = web::Data<Arc<dyn DeleteOAuthState>>;
pub type WebSaveOAuthAccessToken = web::Data<Arc<dyn SaveOAuthAccessToken>>;
pub type WebGetOAuthAccessToken = web::Data<Arc<dyn GetOAuthAccessToken>>;
pub type WebDeleteOAuthAccessToken = web::Data<Arc<dyn DeleteOAuthAccessToken>>;
