// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::borrow::Cow;

use sqlx::Error as SqlxError;

use crate::auth::application::port::out::db::errors::OutDBPortError;

impl From<SqlxError> for OutDBPortError {
    fn from(e: SqlxError) -> Self {
        log::error!("[postgres] err: {}", e);
        if let SqlxError::Database(err) = e {
            if err.code() == Some(Cow::from("23505")) {
                let msg = err.message();
                if msg.contains("oauth_state_state_key") {
                    return Self::DuplicateState;
                } else if msg.contains("oauth_access_token_username_oauth_provider_key") {
                    return Self::DuplicateAccessToken;
                } else {
                    println!("{msg}");
                }
            }
        }
        Self::InternalError
    }
}
