// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

//use crate::auth::application::port::out::db::save_oauth_state::SaveOAuthState;
use std::sync::Arc;

use sqlx::postgres::PgPool;

use crate::db::{migrate::RunMigrations, sqlx_postgres::Postgres};

mod delete_oauth_access_token;
mod delete_oauth_state;
mod errors;
mod get_oauth_access_token;
mod oauth_state_exists;
mod save_oauth_access_token;
mod save_oauth_state;

#[derive(Clone)]
pub struct DBOutPostgresAdapter {
    pool: PgPool,
}

impl DBOutPostgresAdapter {
    pub fn new(pool: PgPool) -> Self {
        Self { pool }
    }

    pub fn migratable(&self) -> Arc<dyn RunMigrations> {
        Arc::new(Postgres::new(self.pool.clone()))
    }
}
