// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::sync::Arc;

use reqwest::Client;
use url::Url;

mod get_username;
mod oauth_auth_req_uri;
mod refresh_access_token;
mod request_access_token;

use crate::auth::application::port::input::forge::refresh_access_token::RefreshAccessTokenInputPort;

#[derive(Clone)]
pub struct Forgejo {
    url: Url,
    client_id: String,
    client_secret: String,
    pub http_client: Client,
    refresh_access_token_input_adapter: Arc<dyn RefreshAccessTokenInputPort>,
}

impl Forgejo {
    pub fn new(
        url: Url,
        client_id: String,
        client_secret: String,
        refresh_access_token_input_adapter: Arc<dyn RefreshAccessTokenInputPort>,
    ) -> Self {
        Self {
            url,
            client_id,
            client_secret,
            http_client: Client::default(),
            refresh_access_token_input_adapter,
        }
    }

    pub fn url(&self) -> &Url {
        &self.url
    }

    pub fn client_id(&self) -> &str {
        &self.client_id
    }

    pub fn client_secret(&self) -> &str {
        &self.client_secret
    }
}
