// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use url::Url;

use super::Forgejo;
use crate::auth::application::port::out::forge::{
    errors::OutForgePortResult, oauth_auth_req_uri::OAuthAuthReqUri,
};

impl OAuthAuthReqUri for Forgejo {
    fn oauth_auth_req_uri(
        &self,
        state: &str,
        process_authorization_response_uri: &Url,
    ) -> OutForgePortResult<Url> {
        let mut u = self.url().to_owned();
        u.set_path("/login/oauth/authorize/");
        u.set_query(Some(&format!(
            "client_id={}&redirect_uri={}&response_type=code&state={state}",
            self.client_id(),
            process_authorization_response_uri.as_str()
        )));

        Ok(u)
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    use crate::auth::application::port::input::forge::refresh_access_token::tests::*;

    #[test]
    fn test_forgejo_oauth_auth_req_uri() {
        let u = Url::parse("https://forgejo").unwrap();
        let client_id = "id";
        let client_secret = "sec";
        let state = "st";
        let process_authorization_response_uri = Url::parse("https://forgeflux").unwrap();

        let f = Forgejo::new(
            u.clone(),
            client_id.into(),
            client_secret.into(),
            mock_refresh_access_token_usecase_factory(None),
        );
        let oauth_auth_req_uri = f
            .oauth_auth_req_uri(state, &process_authorization_response_uri)
            .unwrap();

        let path = "/login/oauth/authorize";
        let q = format!(
            "client_id={}&redirect_uri={}&response_type=code&state={state}",
            client_id,
            process_authorization_response_uri.as_str()
        );
        assert_eq!(
            oauth_auth_req_uri.to_string(),
            format!("https://forgejo{path}/?{q}")
        );
    }
}
