// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::str::FromStr;

use derive_more::Display;
use serde::{Deserialize, Serialize};

pub mod forge_factory;
pub mod forge_repository;
pub mod forgejo;
pub mod github;

#[derive(Debug, Display, Clone, Hash, Serialize, Deserialize, PartialEq, Eq, PartialOrd, Ord)]
#[serde(rename_all = "lowercase")]
pub enum SupportedForges {
    #[display(fmt = "forgejo")]
    Forgejo,
    #[display(fmt = "github")]
    Github,
}

impl FromStr for SupportedForges {
    type Err = SupportedForgesError;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s.trim() {
            "forgejo" => Ok(SupportedForges::Forgejo),
            "github" => Ok(SupportedForges::Github),
            _ => Err(SupportedForgesError::UnsupportedForge),
        }
    }
}

#[derive(Debug, Display, Clone, Hash, Serialize, Deserialize, PartialEq, Eq, PartialOrd, Ord)]
pub enum SupportedForgesError {
    UnsupportedForge,
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_supported_forges() {
        assert_eq!(SupportedForges::Forgejo.to_string(), "forgejo");
        assert_eq!(
            SupportedForges::from_str("forgejo").unwrap(),
            SupportedForges::Forgejo
        );

        assert_eq!(SupportedForges::Github.to_string(), "github");
        assert_eq!(
            SupportedForges::from_str("github").unwrap(),
            SupportedForges::Github
        );
    }
}
