// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use mockall::predicate::*;
use mockall::*;

use super::errors::*;

#[automock]
#[async_trait::async_trait]
pub trait DeleteOAuthAccessToken: Send + Sync {
    async fn delete_oauth_access_token(
        &self,
        username: &str,
        oauth_provider: &str,
    ) -> OutDBPortResult<()>;
}

#[cfg(test)]
pub mod tests {
    use super::*;

    use std::sync::Arc;

    pub fn mock_delete_oauth_access_token(times: Option<usize>) -> Arc<dyn DeleteOAuthAccessToken> {
        let mut m = MockDeleteOAuthAccessToken::default();
        if let Some(times) = times {
            m.expect_delete_oauth_access_token()
                .times(times)
                .returning(move |_, _| Ok(()));
        } else {
            m.expect_delete_oauth_access_token()
                .returning(move |_, _| Ok(()));
        }

        Arc::new(m)
    }
}
