// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use mockall::predicate::*;
use mockall::*;

use super::errors::*;

#[automock]
#[async_trait::async_trait]
pub trait DeleteOAuthState: Send + Sync {
    /// Delete OAuth
    async fn delete_oauth_state(&self, state: &str, oauth_provider: &str) -> OutDBPortResult<()>;
}

#[cfg(test)]
pub mod tests {
    use super::*;

    use std::sync::Arc;

    pub fn mock_delete_oauth_state(times: Option<usize>) -> Arc<dyn DeleteOAuthState> {
        let mut m = MockDeleteOAuthState::default();
        if let Some(times) = times {
            m.expect_delete_oauth_state()
                .times(times)
                .returning(move |_, _| Ok(()));
        } else {
            m.expect_delete_oauth_state().returning(move |_, _| Ok(()));
        }
        Arc::new(m)
    }
}
