// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use mockall::predicate::*;
use mockall::*;

use super::errors::*;

use crate::auth::domain::OAuthAccessToken;

#[automock]
#[async_trait::async_trait]
pub trait GetOAuthAccessToken: Send + Sync {
    async fn get_oauth_access_token(
        &self,
        username: &str,
        oauth_provider: &str,
    ) -> OutDBPortResult<OAuthAccessToken>;
}

#[cfg(test)]
pub mod tests {
    use super::*;

    use std::sync::Arc;

    pub fn mock_get_oauth_access_token(times: Option<usize>) -> Arc<dyn GetOAuthAccessToken> {
        let mut m = MockGetOAuthAccessToken::default();
        if let Some(times) = times {
            m.expect_get_oauth_access_token()
                .times(times)
                .returning(move |_, _| Ok(OAuthAccessToken::default()));
        } else {
            m.expect_get_oauth_access_token()
                .returning(move |_, _| Ok(OAuthAccessToken::default()));
        }

        Arc::new(m)
    }
}
