// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use mockall::predicate::*;
use mockall::*;
use url::Url;

use super::errors::*;

#[automock]
#[async_trait::async_trait]
pub trait SaveOAuthState: Send + Sync {
    /// Save OAuth state code generated during authorization request, which will later be used to
    /// validate authorization response from OAuth server
    async fn save_oauth_state(
        &self,
        state: &str,
        oauth_provider: &str,
        redirect_uri: &Url,
    ) -> OutDBPortResult<()>;
}

#[cfg(test)]
pub mod tests {
    use super::*;

    use std::sync::Arc;

    pub fn mock_save_oauth_state(times: Option<usize>) -> Arc<dyn SaveOAuthState> {
        let mut m = MockSaveOAuthState::new();
        if let Some(times) = times {
            m.expect_save_oauth_state()
                .times(times)
                .returning(|_, _, _| Ok(()));
        } else {
            m.expect_save_oauth_state().returning(|_, _, _| Ok(()));
        }

        Arc::new(m)
    }
}
