// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use mockall::*;
use url::Url;

use super::errors::*;

#[automock]
pub trait OAuthAuthReqUri: Send + Sync {
    fn oauth_auth_req_uri(
        &self,
        state: &str,
        process_authorization_response_uri: &Url,
    ) -> OutForgePortResult<Url>;
}

#[cfg(test)]
pub mod tests {
    use super::*;

    use std::sync::Arc;

    pub fn mock_oauth_auth_req_uri(
        times: Option<usize>,
        returning: Url,
    ) -> Arc<dyn OAuthAuthReqUri> {
        let mut m = MockOAuthAuthReqUri::new();
        if let Some(times) = times {
            m.expect_oauth_auth_req_uri()
                .times(times)
                .returning(move |_, _| Ok(returning.clone()));
        } else {
            m.expect_oauth_auth_req_uri()
                .returning(move |_, _| Ok(returning.clone()));
        }

        Arc::new(m)
    }
}
