// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use mockall::predicate::*;
use mockall::*;

use super::errors::*;
use crate::auth::domain::OAuthAccessToken;

#[automock]
#[async_trait::async_trait]
pub trait RefreshAccessToken: Send + Sync {
    async fn refresh_access_token(
        &self,
        old_access_token: &OAuthAccessToken,
    ) -> OutForgePortResult<OAuthAccessToken>;
}

#[cfg(test)]
pub mod tests {
    use super::*;

    use std::sync::Arc;

    pub fn mock_refresh_access_token(times: Option<usize>) -> Arc<dyn RefreshAccessToken> {
        let mut m = MockRefreshAccessToken::new();
        if let Some(times) = times {
            m.expect_refresh_access_token()
                .times(times)
                .returning(move |_| Ok(OAuthAccessToken::default()));
        } else {
            m.expect_refresh_access_token()
                .returning(move |_| Ok(OAuthAccessToken::default()));
        }

        Arc::new(m)
    }
}
