// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use derive_more::Display;
use serde::{Deserialize, Serialize};

use crate::auth::application::port::out::db::errors::OutDBPortError;
use crate::auth::application::port::out::forge::errors::OutForgePortError;

pub type ProcessAuthorizationServiceResult<V> = Result<V, ProcessAuthorizationServiceError>;

#[derive(Debug, Display, Clone, Serialize, Deserialize, PartialEq, Eq, PartialOrd, Ord)]
pub enum ProcessAuthorizationServiceError {
    InternalError,
    BadRequest,
}

impl From<OutDBPortError> for ProcessAuthorizationServiceError {
    fn from(v: OutDBPortError) -> Self {
        match v {
            OutDBPortError::DuplicateState => Self::InternalError, // only happens when there's a
            // bug in code
            OutDBPortError::InternalError => Self::InternalError,
            OutDBPortError::DuplicateAccessToken => Self::InternalError, // only happens when bug in
                                                                         // code
        }
    }
}

impl From<OutForgePortError> for ProcessAuthorizationServiceError {
    fn from(v: OutForgePortError) -> Self {
        match v {
            OutForgePortError::InternalError => Self::InternalError,
            OutForgePortError::OAuthAccessTokenExpired => unimplemented!(),
        }
    }
}
