// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use serde::{Deserialize, Serialize};

use super::errors::*;

#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, PartialOrd, Ord)]
pub struct RefreshAccessTokenCommand {
    username: String,
    oauth_provider: String,
}

impl RefreshAccessTokenCommand {
    pub fn username(&self) -> &str {
        &self.username
    }
    pub fn oauth_provider(&self) -> &str {
        &self.oauth_provider
    }

    pub fn new_command(
        username: String,
        oauth_provider: String,
    ) -> RefreshAccessTokenServiceResult<Self> {
        let username = username.trim().to_string();
        let oauth_provider = oauth_provider.trim().to_string();
        Ok(Self {
            username,
            oauth_provider,
        })
    }
}
