// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use mockall::predicate::*;
use mockall::*;

pub mod command;
pub mod errors;
pub mod service;

#[automock]
#[async_trait::async_trait]
pub trait RefreshAccessTokenUseCase: Send + Sync {
    async fn refresh_access_token(
        &self,
        cmd: command::RefreshAccessTokenCommand,
    ) -> errors::RefreshAccessTokenServiceResult<()>;
}

#[cfg(test)]
pub mod tests {
    use std::sync::Arc;

    use super::*;

    pub fn mock_refresh_access_token_usecase(
        times: Option<usize>,
    ) -> Arc<dyn RefreshAccessTokenUseCase> {
        let mut m = MockRefreshAccessTokenUseCase::new();
        if let Some(times) = times {
            m.expect_refresh_access_token()
                .times(times)
                .returning(move |_| Ok(()));
        } else {
            m.expect_refresh_access_token().returning(move |_| Ok(()));
        }

        Arc::new(m)
    }
}
