// SPDX-FileCopyrightText: 2024 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use time::{Duration, OffsetDateTime};

use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Serialize, Deserialize, Eq, PartialOrd, Ord)]
pub struct OAuthAccessToken {
    //pub username: String,
    pub refresh_token: String,
    pub access_token: String,
    pub token_type: String,
    pub expires_at: OffsetDateTime, // unix epoch of the instant
    pub created_at: OffsetDateTime, // unix epoch of the instant
    pub expires_in: u64,
}

#[cfg(test)]
impl Default for OAuthAccessToken {
    fn default() -> Self {
        let now = OffsetDateTime::now_utc();
        Self {
            refresh_token: String::default(),
            access_token: String::default(),
            token_type: String::default(),
            expires_at: now, // unix epoch of the instant
            created_at: now, // unix epoch of the instant
            expires_in: u64::default(),
        }
    }
}

impl PartialEq for OAuthAccessToken {
    fn eq(&self, other: &OAuthAccessToken) -> bool {
        self.refresh_token == other.refresh_token
            && self.access_token == other.access_token
            && self.token_type == other.token_type
            && self.expires_at.unix_timestamp() == other.expires_at.unix_timestamp()
            && self.created_at.unix_timestamp() == other.created_at.unix_timestamp()
            && self.expires_in == other.expires_in
    }
}

impl OAuthAccessToken {
    pub fn new(
        //   username: String,
        refresh_token: String,
        access_token: String,
        token_type: String,
        expires_in: u64,
        created_at: OffsetDateTime,
    ) -> Self {
        let mut expires_at = created_at;

        expires_at = expires_at
            .checked_add(Duration::seconds(expires_in as i64))
            .unwrap();
        Self {
            //      username,
            refresh_token,
            access_token,
            token_type,
            expires_in,
            expires_at,
            created_at,
        }
    }
}
